@@ -66,6 +66,7 @@ USERURL_FID        = "WXXX";
 PLAYCOUNT_FID      = "PCNT";
 UNIQUE_FILE_ID_FID = "UFID";
 BPM_FID            = "TBPM";
+PUBLISHER_FID      = "TPUB";
 
 obsoleteFrames = {"EQUA": "Equalisation",
                   "IPLS": "Involved people list",
@@ -304,7 +305,10 @@ def id3EncodingToString(encoding):
     elif encoding == UTF_16BE_ENCODING:
         return "utf_16_be";
     else:
-        raise ValueError;
+        if strictID3():
+            raise ValueError;
+        else:
+            return "latin_1";
 
 ################################################################################
 class FrameException(Exception):
@@ -360,6 +364,11 @@ class FrameHeader:
       else:
          self.majorVersion = tagHeader.majorVersion;
          self.minorVersion = tagHeader.minorVersion;
+      # Correctly set size of header
+      if self.minorVersion == 2:
+         self.FRAME_HEADER_SIZE = 6;
+      else:
+         self.FRAME_HEADER_SIZE = 10;
       self.setBitMask();
 
    def setBitMask(self):
@@ -423,13 +432,13 @@ class FrameHeader:
       return data;
 
    def parse2_2(self, f):
-      frameId = f.read(3);
-      frameId = map2_2FrameId(frameId);
+      frameId_22 = f.read(3);
+      frameId = map2_2FrameId(frameId_22);
       if self.isFrameIdValid(frameId):
-         TRACE_MSG("FrameHeader [id]: %s (0x%x%x%x)" % (frameId,
-                                                       ord(frameId[0]),
-                                                       ord(frameId[1]),
-                                                       ord(frameId[2])));
+         TRACE_MSG("FrameHeader [id]: %s (0x%x%x%x)" % (frameId_22,
+                                                        ord(frameId_22[0]),
+                                                        ord(frameId_22[1]),
+                                                        ord(frameId_22[2])));
          self.id = frameId;
          # dataSize corresponds to the size of the data segment after
          # encryption, compression, and unsynchronization.
@@ -579,10 +588,10 @@ class Frame:
       return data;
 
    def decrypt(self, data):
-      raise FrameException("Ecnyption Not Supported");
+      raise FrameException("Encryption not supported");
 
    def encrypt(self, data):
-      raise FrameException("Ecnyption Not Supported");
+      raise FrameException("Encryption not supported");
 
    def disassembleFrame(self, data):
       # Format flags in the frame header may add extra data to the
@@ -985,14 +994,14 @@ class UserURLFrame(URLFrame):
 
 ################################################################################
 class CommentFrame(Frame):
-   lang = u"";
+   lang = "";
    description = u"";
    comment = u"";
 
    # Data string format:
    # encoding (one byte) + lang (three byte code) + description + "\x00" +
    # text
-   def __init__(self, frameHeader, data = None, lang = u"",
+   def __init__(self, frameHeader, data = None, lang = "",
                 description = u"", comment = u"", encoding = DEFAULT_ENCODING):
        Frame.__init__(self, frameHeader);
        if data != None:
@@ -1169,11 +1178,16 @@ class ImageFrame(Frame):
       self.encoding = input.read(1);
       TRACE_MSG("APIC encoding: " + id3EncodingToString(self.encoding));
 
+      # Mime type
       self.mimeType = "";
-      ch = input.read(1);
-      while ch != "\x00":
-         self.mimeType += ch;
-         ch = input.read(1);
+      if self.header.minorVersion != 2:
+          ch = input.read(1);
+          while ch != "\x00":
+              self.mimeType += ch;
+              ch = input.read(1);
+      else:
+          # v2.2 (OBSOLETE) special case
+          self.mimeType = input.read(3);
       TRACE_MSG("APIC mime type: " + self.mimeType);
       if strictID3() and not self.mimeType:
          raise FrameException("APIC frame does not contain a mime type");
@@ -1230,8 +1244,10 @@ class ImageFrame(Frame):
       f.write(self.imageData);
       f.flush();
       f.close();
-   def getDefaultFileName(self):
+   def getDefaultFileName(self, suffix = ""):
       nameStr = self.picTypeToString(self.pictureType);
+      if suffix:
+          nameStr += suffix;
       nameStr = nameStr +  "." + self.mimeType.split("/")[1];
       return nameStr;
 
@@ -1499,7 +1515,7 @@ class FrameSet(list):
 
          self.addFrame(createFrame(frameHeader, data));
 
-         # Each frame contains dataSize + headerSize(10) bytes.
+         # Each frame contains dataSize + headerSize bytes.
          sizeLeft -= (frameHeader.FRAME_HEADER_SIZE + frameHeader.dataSize);
 
       return paddingSize;
@@ -1524,12 +1540,12 @@ class FrameSet(list):
       # No multiples except for TXXX which must have unique descriptions.
       if strictID3() and TEXT_FRAME_RX.match(fid) and self[fid]:
          if not USERTEXT_FRAME_RX.match(fid):
-            raise FrameException("Multiple %s frames now allowed." % fid);
+            raise FrameException("Multiple %s frames not allowed." % fid);
          userTextFrames = self[fid];
          for frm in userTextFrames:
             if frm.description == frame.description:
                raise FrameException("Multiple %s frames with the same\
-                                     description now allowed." % fid);
+                                     description not allowed." % fid);
 
       # Comment frame restrictions.
       # Multiples must have a unique description/language combination.
@@ -1539,38 +1555,38 @@ class FrameSet(list):
             if frm.description == frame.description and\
                frm.lang == frame.lang:
                raise FrameException("Multiple %s frames with the same\
-                                     language and description now allowed." %\
+                                     language and description not allowed." %\
                                      fid);
 
       # URL frame restrictions.
       # No multiples except for TXXX which must have unique descriptions.
       if strictID3() and URL_FRAME_RX.match(fid) and self[fid]:
          if not USERURL_FRAME_RX.match(fid):
-            raise FrameException("Multiple %s frames now allowed." % fid);
+            raise FrameException("Multiple %s frames not allowed." % fid);
          userUrlFrames = self[fid];
          for frm in userUrlFrames:
             if frm.description == frame.description:
                raise FrameException("Multiple %s frames with the same\
-                                     description now allowed." % fid);
+                                     description not allowed." % fid);
 
       # Music CD ID restrictions.
       # No multiples.
       if strictID3() and CDID_FRAME_RX.match(fid) and self[fid]:
-         raise FrameException("Multiple %s frames now allowed." % fid);
+         raise FrameException("Multiple %s frames not allowed." % fid);
 
       # Image (attached picture) frame restrictions.
       # Multiples must have a unique content desciptor.  I'm assuming that
       # the spec means the picture type.....
-      if IMAGE_FRAME_RX.match(fid) and self[fid]:
+      if IMAGE_FRAME_RX.match(fid) and self[fid] and strictID3():
          imageFrames = self[fid];
          for frm in imageFrames:
             if frm.pictureType == frame.pictureType:
                raise FrameException("Multiple %s frames with the same "\
-                                    "content descriptor now allowed." % fid);
+                                    "content descriptor not allowed." % fid);
 
       # Play count frame (PCNT).  There may be only one
       if PLAYCOUNT_FRAME_RX.match(fid) and self[fid]:
-         raise FrameException("Multiple %s frames now allowed." % fid);
+         raise FrameException("Multiple %s frames not allowed." % fid);
 
       # Unique File identifier frame.  There may be only one with the same
       # owner_id
@@ -1578,7 +1594,7 @@ class FrameSet(list):
           ufid_frames = self[fid];
           for frm in ufid_frames:
               if frm.owner_id == frame.owner_id:
-                  raise FrameException("Multiple %s frames now allowed with "\
+                  raise FrameException("Multiple %s frames not allowed with "\
                                        "the same owner ID (%s)" %\
                                        (fid, frame.owner_id));
 
