@@ -392,7 +392,7 @@ class Tag:
       self.header = TagHeader();
       self.frames = FrameSet(self.header);
       self.iterIndex = None;
-      linkedFile = None;
+      self.linkedFile = None;
 
    # Returns an read-only iterator for all frames.
    def __iter__(self):
@@ -478,7 +478,7 @@ class Tag:
       version = self.getVersion();
       if version == ID3_V2_2:
           raise TagException("Unable to write ID3 v2.2");
-      # If v1.0 is being for explicitly then so be it, if not and there is
+      # If v1.0 is being requested explicitly then so be it, if not and there is
       # a track number then bumping to v1.1 is /probably/ best.
       if self.header.majorVersion == 1 and self.header.minorVersion == 0 and\
          self.getTrackNum()[0] != None and version != ID3_V1_0:
@@ -894,6 +894,14 @@ class Tag:
    def setBPM(self, bpm):
        self.setTextFrame(BPM_FID, self.strToUnicode(str(bpm)));
 
+   def getPublisher(self):
+      pub = self.frames[PUBLISHER_FID];
+      if pub:
+          return pub[0].text or None;
+
+   def setPublisher(self, p):
+       self.setTextFrame(PUBLISHER_FID, self.strToUnicode(str(p)));
+
    # Test ID3 major version.
    def isV1(self):
       return self.header.majorVersion == 1;
@@ -1223,8 +1231,9 @@ class Tag:
          padding = self.frames.parse(fp, self.header);
          TRACE_MSG("Tag contains %d bytes of padding." % padding);
       except FrameException, ex:
-         fp.close();
-         raise TagException(str(ex));
+         if utils.strictID3():
+            fp.close();
+            raise TagException(str(ex));
       except TagException:
          fp.close();
          raise;
@@ -1423,6 +1432,8 @@ class TagFile:
    fileName = str("");
    fileSize = int(0);
    tag      = None;
+   # Number of seconds required to play the audio file.
+   play_time = int(0);
 
    def __init__(self, fileName):
        self.fileName = fileName;
@@ -1439,6 +1450,8 @@ class TagFile:
        base = os.path.basename(self.fileName);
        base_ext = os.path.splitext(base)[1];
        dir = os.path.dirname(self.fileName);
+       if not dir:
+           dir = ".";
        new_name = dir + os.sep + name.encode("iso8859-1") + base_ext;
        try:
            os.rename(self.fileName, new_name);
@@ -1447,20 +1460,30 @@ class TagFile:
            raise TagException("Error renaming '%s' to '%s'" % (self.fileName,
                                                                new_name));
 
+   def getPlayTime(self):
+      return self.play_time;
+
+   def getPlayTimeString(self):
+      total = self.getPlayTime();
+      h = total / 3600;
+      m = (total % 3600) / 60;
+      s = (total % 3600) % 60;
+      if h:
+         timeStr = "%d:%.2d:%.2d" % (h, m, s);
+      else:
+         timeStr = "%d:%.2d" % (m, s);
+      return timeStr;
+
 
 ################################################################################
 class Mp3AudioFile(TagFile):
    header         = mp3.Header();
    xingHeader     = None;
    invalidFileExc = InvalidAudioFormatException("File is not mp3");
-   # Number of seconds required to play the audio file.
-   playTime       = None;
 
    def __init__(self, fileName, tagVersion = ID3_ANY_VERSION):
       TagFile.__init__(self, fileName);
 
-      self.playTime = None;
-
       if not isMp3File(fileName):
          raise self.invalidFileExc;
 
@@ -1516,7 +1539,7 @@ class Mp3AudioFile(TagFile):
       # Compute track play time.
       tpf = mp3.computeTimePerFrame(header);
       if xingHeader:
-         self.playTime = int(tpf * xingHeader.numFrames);
+         self.play_time = int(tpf * xingHeader.numFrames);
       else:
          length = self.getSize();
          if tag and tag.isV2():
@@ -1527,27 +1550,13 @@ class Mp3AudioFile(TagFile):
                length -= 128;
          elif tag and tag.isV1():
             length -= 128;
-         self.playTime = int((length / header.frameLength) * tpf);    
+         self.play_time = int((length / header.frameLength) * tpf);    
 
       self.header = header;
       self.xingHeader = xingHeader;
       self.tag = tag;
       f.close();
 
-   def getPlayTime(self):
-      return self.playTime;
-
-   def getPlayTimeString(self):
-      total = self.getPlayTime();
-      h = total / 3600;
-      m = (total % 3600) / 60;
-      s = (total % 3600) % 60;
-      if h:
-         timeStr = "%d:%.2d:%.2d" % (h, m, s);
-      else:
-         timeStr = "%d:%.2d" % (m, s);
-      return timeStr;
-
    # Returns a tuple.  The first value is a boolean which if true means the
    # bit rate returned in the second value is variable.
    def getBitRate(self):
